import { Class, Annotation } from '@overflow/commons/core/reflect';
import { ClassType } from '@overflow/commons/core/type';

export abstract class AnnotationUtils {
  public static hasAnnotation<T extends Annotation>(classType: ClassType, annotationClass: ClassType<T>): boolean {
    let annotation = AnnotationUtils.getAnnotation(classType, annotationClass);
    if (undefined !== annotation) {
      return true;
    }
    return false;
  }

  public static getAnnotation<T extends Annotation>(classType: ClassType, annotationClass: ClassType<T>): T | undefined {
    let clazz = Class.forClassType(classType);
    if (undefined === clazz) {
      return undefined;
    }

    let annotations = clazz.getAnnotations();
    if (0 === annotations.size) {
      return undefined;
    }

    for (const annonClassType of Array.from(annotations.keys())) {
      if (annonClassType === annotationClass) {
        return <T>annotations.get(annonClassType);
      }
      let annotation = AnnotationUtils.getAnnotation(annonClassType, annotationClass);
      if (undefined !== annotation) {
        return annotation;
      }
    }

    return undefined;
  }



}
