import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';

export interface InjectAnnotationAttributes {
  required?: boolean;
}

export class InjectAnnotation extends Annotation {
  public readonly attributes: InjectAnnotationAttributes = {
    required: true,
  };

  public constructor(required?: boolean | InjectAnnotationAttributes) {
    super();

    if (undefined === required) {
      return;
    }

    if (TypeUtil.isBoolean(required)) {
      this.attributes.required = <boolean>required;
    } else {
      this.copyAttributes(this.attributes, <InjectAnnotationAttributes>required);
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Inject');
  }

  public propertyDecorator(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Inject');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Inject');
  }

  public parameterDecorator(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Inject');
  }

}

export const Inject = Decorator.create(InjectAnnotation);
